//// D O C U M E N T A T I O N ////////////////////////////////////////////
// PROGRAMMER:  Michael Tese
//              CompuServe 71413,2632
// PROJECT:     3D OO Starfield
// START DATE:  5/30/95
// DESCRIPTION: Programmed in Turbo C Version 3.0
//              Can be compiled under the small model
//              Original Idea from:
//                 Mark Bryant
//                 CompuServe 100044.256
//************************************************************************


// I N C L U D E S ///////////////////////////////////////////////////////
#include <stdlib.h>
#include <conio.h>


// C O N S T S ///////////////////////////////////////////////////////////
const int TOTAL_STARS=500;
const int SPEED=2;
const int VGA256=0x13;
const int TEXT_MODE=0x03;
const int VGA_VSYNC_MASK=0x08;
const int VGA_INPUT_STATUS_1=0x3DA;


// G L O B A L S /////////////////////////////////////////////////////////
char far *pixel;    // far pointer to video memory


// C L A S S E S ///////////////////////////////////////////////////////
class Star {
  public:
   Star(void) {
      spx=(random(640)-320) * 100;
      spy=(random(400)-200) * 100;
      spz=random(510);
   }

   void Plot_New_Star(void) {
      spx =(random(640)-320) * 100;
      spy =(random(400)-200) * 100;
      spz =510;
   } //end Plot_New_Star

   void DecreaseZ() {
      spz=spz-SPEED;
      if (spz<=0) Plot_New_Star();
   } //end DecreaseZ

   void Calc_Screen_Pos(void) {
      px=(spx/spz)+160;    // calculate the screen position
      py=(spy/spz)+100;    // with an offset of the screen centre

      if (px<0 || px>319)   Plot_New_Star();
      if (py<0 || py>199)   Plot_New_Star();
   } //end Calc_Screen_Pos

   void Plot_Star(void) {
      pixel = (char far *)0xA0000000 + py * 320 + px;
      pixel[0] = spz / 2;  // plot the new star with a colour
   } //end Plot_Star

   void Delete_Old_Star(void) {
      pixel = (char far *)0xA0000000 + py * 320 + px;
      pixel[0] = 0;                 // clear the old position
   } //end Delete_Old_Star

  protected:
   int spx;
   int spy;
   int spz;
   int px;
   int py;

};


// P R O T O T Y P E S ///////////////////////////////////////////////////
void Set_Mode(int mode);
void Wait_For_Vsync(void);
void Define_Pallete(void);
Get_Scan_Code(void);


// M A I N ///////////////////////////////////////////////////////////////
int main(void) {
   int done=0;
   Star star[TOTAL_STARS];

   Set_Mode(VGA256);     // Change the video mode to 320x200 256 colors
   Define_Pallete();

   do {
      // If keyboard events requested....
      unsigned char userinput = Get_Scan_Code();
      if (userinput == 1) done=1;

      Wait_For_Vsync();  // Wait for the vertical retrace

      for (int count=0 ; count<TOTAL_STARS ; count++) {

	  star[count].Delete_Old_Star();

	  star[count].DecreaseZ();

	  star[count].Calc_Screen_Pos();

	  star[count].Plot_Star();

      } //endfor

   } while (!done);

   Set_Mode(TEXT_MODE);
   return(done);

} //end main


// N O N - M E M B E R  F U N C T I O N S ////////////////////////////////
void Set_Mode(int mode)
{
   asm {
    mov ax,mode
    int 0x010
   }
} //end Set_Mode


//////////////////////////////////////////////////////////////////////////
void Wait_For_Vsync(void )
{
   while(!(inp(VGA_INPUT_STATUS_1) & VGA_VSYNC_MASK)) {
     // do nothing, wait for start of retrace
   } // endwhile
} //end Wait_For_Vsync


//////////////////////////////////////////////////////////////////////////
void Define_Pallete(void) {
   // define the pallete of greys
   // color 1 is white and gradually fading to black at colour 255
   int a=63;
   int b=4;
   for (int x=1;x<=255;x++) {
      b=b-1;
      if (b==0) { a=a-1; b=4; }
      outportb(0x3c8,x);
      outportb(0x3c9,a);
      outportb(0x3c9,a);
      outportb(0x3c9,a);
   } //endfor
} //end Define_Pallete


//////////////////////////////////////////////////////////////////////////
Get_Scan_Code(void) {

// is a key ready?
  asm mov AH,0x01;         // function 1: is a key ready?
  asm int 0x16;            // call the interrupt
  asm jz empty;            // there was no key so exit
  asm mov AH,0x00;         // function 0: get the scan code please
  asm int 0x16;            // call the interrupt
  asm mov AL,AH;           // result was in ah so put into al
  asm xor AH,AH;           // zero out ah
  asm jmp done;            // data's in ax...let's blaze!

empty:
  asm xor AX,AX;          // clear out ax i.e. 0 means no key

done:

// since data is in ax it will be returned properly
} //end Get_Scan_Code
